@@ -18,6 +18,8 @@
  *              Marco Trevisan (Treviño) <3v1n0@ubuntu.com>
  */
 
+#include <boost/algorithm/string.hpp>
+
 #include <Nux/Nux.h>
 #include <Nux/BaseWindow.h>
 
@@ -41,6 +43,11 @@ namespace unity
 namespace launcher
 {
 
+namespace
+{
+  const std::string DEFAULT_ICON = "application-default-icon";
+}
+
 NUX_IMPLEMENT_OBJECT_TYPE(BamfLauncherIcon);
 
 BamfLauncherIcon::BamfLauncherIcon(BamfApplication* app)
@@ -52,12 +59,14 @@ BamfLauncherIcon::BamfLauncherIcon(BamfApplication* app)
   , _fill_supported_types_id(0)
   , _window_moved_id(0)
 {
+  g_object_set_qdata(G_OBJECT(app), g_quark_from_static_string("unity-seen"),
+                     GUINT_TO_POINTER(1));
   auto bamf_view = glib::object_cast<BamfView>(_bamf_app);
 
   glib::String icon(bamf_view_get_icon(bamf_view));
 
   tooltip_text = BamfName();
-  icon_name = icon.Str();
+  icon_name = (icon ? icon.Str() : DEFAULT_ICON);
   SetIconType(TYPE_APPLICATION);
 
   if (IsSticky())
@@ -71,7 +80,7 @@ BamfLauncherIcon::BamfLauncherIcon(BamfApplication* app)
   glib::SignalBase* sig;
 
   sig = new glib::Signal<void, BamfView*, BamfView*>(bamf_view, "child-added",
-                          [&] (BamfView*, BamfView*) {
+                          [this] (BamfView*, BamfView*) {
                             EnsureWindowState();
                             UpdateMenus();
                             UpdateIconGeometries(GetCenters());
@@ -79,26 +88,46 @@ BamfLauncherIcon::BamfLauncherIcon(BamfApplication* app)
   _gsignals.Add(sig);
 
   sig = new glib::Signal<void, BamfView*, BamfView*>(bamf_view, "child-removed",
-                          [&] (BamfView*, BamfView*) { EnsureWindowState(); });
+                          [this] (BamfView*, BamfView*) { EnsureWindowState(); });
   _gsignals.Add(sig);
 
   sig = new glib::Signal<void, BamfView*, gboolean>(bamf_view, "urgent-changed",
-                          [&] (BamfView*, gboolean urgent) {
+                          [this] (BamfView*, gboolean urgent) {
                             SetQuirk(QUIRK_URGENT, urgent);
                           });
   _gsignals.Add(sig);
 
   sig = new glib::Signal<void, BamfView*, gboolean>(bamf_view, "active-changed",
-                          [&] (BamfView*, gboolean active) {
+                          [this] (BamfView*, gboolean active) {
                             SetQuirk(QUIRK_ACTIVE, active);
                           });
   _gsignals.Add(sig);
 
   sig = new glib::Signal<void, BamfView*, gboolean>(bamf_view, "running-changed",
-                          [&] (BamfView*, gboolean running) {
+                          [this] (BamfView* view, gboolean running) {
                             SetQuirk(QUIRK_RUNNING, running);
+
                             if (running)
                             {
+                              if (_remove_timeout_id)
+                              {
+                                g_source_remove(_remove_timeout_id);
+                                _remove_timeout_id = 0;
+                              }
+
+                              /* It can happen that these values are not set
+                               * during initialization if the view is closed
+                               * very early, so we need to make sure that they
+                               * are updated as soon as the view is re-opened. */
+                              if (tooltip_text().empty())
+                                tooltip_text = BamfName();
+
+                              if (icon_name == DEFAULT_ICON)
+                              {
+                                glib::String icon(bamf_view_get_icon(view));
+                                icon_name = (icon ? icon.Str() : DEFAULT_ICON);
+                              }
+
                               EnsureWindowState();
                               UpdateIconGeometries(GetCenters());
                             }
@@ -106,16 +135,32 @@ BamfLauncherIcon::BamfLauncherIcon(BamfApplication* app)
   _gsignals.Add(sig);
 
   sig = new glib::Signal<void, BamfView*, gboolean>(bamf_view, "user-visible-changed",
-                          [&] (BamfView*, gboolean visible) {
+                          [this] (BamfView*, gboolean visible) {
                             if (!IsSticky())
                               SetQuirk(QUIRK_VISIBLE, visible);
                           });
   _gsignals.Add(sig);
 
   sig = new glib::Signal<void, BamfView*>(bamf_view, "closed",
-                          [&] (BamfView*) {
+                          [this] (BamfView*) {
                             if (!IsSticky())
-                              Remove();
+                            {
+                              /* Use a timeout to remove the icon, this avoids
+                               * that we remove an application that is going
+                               * to be reopened soon. So applications that
+                               * have a splash screen won't be removed from
+                               * the launcher while the splash is closed and
+                               * a new window is opened. */
+                              if (_remove_timeout_id)
+                                g_source_remove(_remove_timeout_id);
+
+                              _remove_timeout_id = g_timeout_add_seconds(1, [] (gpointer data) -> gboolean {
+                                auto self = static_cast<BamfLauncherIcon*>(data);
+                                self->Remove();
+                                self->_remove_timeout_id = 0;
+                                return false;
+                              }, this);
+                            }
                           });
   _gsignals.Add(sig);
 
@@ -141,12 +186,17 @@ BamfLauncherIcon::BamfLauncherIcon(BamfApplication* app)
 BamfLauncherIcon::~BamfLauncherIcon()
 {
   g_object_set_qdata(G_OBJECT(_bamf_app.RawPtr()),
-                     g_quark_from_static_string("unity-seen"),
-                     GINT_TO_POINTER(0));
+                     g_quark_from_static_string("unity-seen"), nullptr);
+
+  if (_remove_timeout_id != 0)
+    g_source_remove(_remove_timeout_id);
 
   if (_fill_supported_types_id != 0)
     g_source_remove(_fill_supported_types_id);
 
+  if (_quicklist_activated_id != 0)
+    g_source_remove(_quicklist_activated_id);
+
   if (_window_moved_id != 0)
     g_source_remove(_window_moved_id);
 
@@ -154,6 +204,21 @@ BamfLauncherIcon::~BamfLauncherIcon()
     g_source_remove(_dnd_hover_timer);
 }
 
+void BamfLauncherIcon::Remove()
+{
+  /* Removing the unity-seen flag to the wrapped bamf application, on remove
+   * request we make sure that if the bamf application is re-opened while
+   * the removal process is still ongoing, the application will be shown
+   * on the launcher. Disconnecting from signals and nullifying the _bamf_app
+   * we make sure that this icon won't be reused (no duplicated icon). */
+  _gsignals.Disconnect(_bamf_app);
+  g_object_set_qdata(G_OBJECT(_bamf_app.RawPtr()),
+                     g_quark_from_static_string("unity-seen"), nullptr);
+  _bamf_app = nullptr;
+
+  SimpleLauncherIcon::Remove();
+}
+
 bool BamfLauncherIcon::IsSticky() const
 {
   return bamf_view_is_sticky(BAMF_VIEW(_bamf_app.RawPtr()));
@@ -182,15 +247,15 @@ bool BamfLauncherIcon::IsUrgent() const
 void BamfLauncherIcon::ActivateLauncherIcon(ActionArg arg)
 {
   SimpleLauncherIcon::ActivateLauncherIcon(arg);
-  bool scaleWasActive = WindowManager::Default()->IsScaleActive();
-  GList    *l;
+  WindowManager* wm = WindowManager::Default();
+  bool scaleWasActive = wm->IsScaleActive();
 
   bool active = IsActive();
   bool user_visible = IsRunning();
 
   if (arg.target && OwnsWindow(arg.target))
   {
-    WindowManager::Default()->Activate(arg.target);
+    wm->Activate(arg.target);
     return;
   }
 
@@ -203,31 +268,59 @@ void BamfLauncherIcon::ActivateLauncherIcon(ActionArg arg)
     auto bamf_view = glib::object_cast<BamfView>(_bamf_app);
     user_visible = bamf_view_user_visible(bamf_view);
 
-    bool any_visible = false;
-    GList *children = bamf_view_get_children(bamf_view);
-
-    for (l = children; l; l = l->next)
+    if (active)
     {
-      if (!BAMF_IS_WINDOW(l->data))
-        continue;
+      bool any_visible = false;
+      bool any_mapped = false;
+      bool any_on_top = false;
+      bool any_on_monitor = (arg.monitor < 0);
+      int active_monitor = arg.monitor;
+      GList* children = bamf_view_get_children(bamf_view);
 
-      Window xid = bamf_window_get_xid(static_cast<BamfWindow*>(l->data));
-
-      if (!any_visible && WindowManager::Default()->IsWindowOnCurrentDesktop(xid))
+      for (GList* l = children; l; l = l->next)
       {
-        any_visible = true;
+        if (!BAMF_IS_WINDOW(l->data))
+          continue;
+
+        auto view = static_cast<BamfView*>(l->data);
+        auto win = static_cast<BamfWindow*>(l->data);
+        Window xid = bamf_window_get_xid(win);
+
+        if (!any_visible && wm->IsWindowOnCurrentDesktop(xid))
+        {
+          any_visible = true;
+        }
+
+        if (!any_mapped && wm->IsWindowMapped(xid))
+        {
+          any_mapped = true;
+        }
+
+        if (!any_on_top && wm->IsWindowOnTop(xid))
+        {
+          any_on_top = true;
+        }
+
+        if (!any_on_monitor && bamf_window_get_monitor(win) == arg.monitor &&
+            wm->IsWindowMapped(xid) && wm->IsWindowVisible(xid))
+        {
+          any_on_monitor = true;
+        }
+
+        if (bamf_view_is_active(view))
+        {
+          active_monitor = bamf_window_get_monitor(win);
+        }
       }
 
-      if (active && !WindowManager::Default()->IsWindowMapped(xid))
-      {
+      g_list_free(children);
+
+      if (!any_visible || !any_mapped || !any_on_top)
+        active = false;
+
+      if (any_on_monitor && arg.monitor >= 0 && active_monitor != arg.monitor)
         active = false;
-      }
     }
-
-    g_list_free(children);
-
-    if (!any_visible)
-      active = false;
   }
 
   /* Behaviour:
@@ -245,7 +338,7 @@ void BamfLauncherIcon::ActivateLauncherIcon(ActionArg arg)
 
     if (scaleWasActive)
     {
-      WindowManager::Default()->TerminateScale();
+      wm->TerminateScale();
     }
 
     SetQuirk(QUIRK_STARTING, true);
@@ -257,7 +350,7 @@ void BamfLauncherIcon::ActivateLauncherIcon(ActionArg arg)
     {
       if (scaleWasActive) // #5 above
       {
-        WindowManager::Default()->TerminateScale();
+        wm->TerminateScale();
         Focus(arg);
       }
       else // #2 above
@@ -272,7 +365,7 @@ void BamfLauncherIcon::ActivateLauncherIcon(ActionArg arg)
     {
       if (scaleWasActive) // #4 above
       {
-        WindowManager::Default()->TerminateScale();
+        wm->TerminateScale();
         Focus(arg);
         if (arg.source != ActionArg::SWITCHER)
           Spread(true, 0, false);
@@ -290,9 +383,9 @@ void BamfLauncherIcon::ActivateLauncherIcon(ActionArg arg)
 
 std::vector<Window> BamfLauncherIcon::GetWindows(WindowFilterMask filter, int monitor)
 {
+  WindowManager* wm = WindowManager::Default();
   std::vector<Window> results;
   GList* children, *l;
-  WindowManager *wm = WindowManager::Default();
 
   monitor = (filter & WindowFilter::ON_ALL_MONITORS) ? -1 : monitor;
   bool mapped = (filter & WindowFilter::MAPPED);
@@ -467,6 +560,7 @@ void BamfLauncherIcon::AddProperties(GVariantBuilder* builder)
   variant::BuilderWrapper(builder)
     .add("desktop_file", DesktopFile())
     .add("desktop_id", GetDesktopID())
+    .add("application_id", GPOINTER_TO_UINT(_bamf_app.RawPtr()))
     .add("xids", g_variant_builder_end(&xids_builder))
     .add("sticky", IsSticky());
 }
@@ -546,17 +640,16 @@ void BamfLauncherIcon::OpenInstanceLauncherIcon(ActionArg arg)
 
 void BamfLauncherIcon::Focus(ActionArg arg)
 {
-  GList* children, *l;
   bool any_urgent = false;
   bool any_visible = false;
   bool any_user_visible = false;
-
-  children = bamf_view_get_children(BAMF_VIEW(_bamf_app.RawPtr()));
+  WindowManager* wm = WindowManager::Default();
 
   std::vector<Window> windows;
+  GList* children = bamf_view_get_children(BAMF_VIEW(_bamf_app.RawPtr()));
 
   /* get the list of windows */
-  for (l = children; l; l = l->next)
+  for (GList* l = children; l; l = l->next)
   {
     if (!BAMF_IS_WINDOW(l->data))
       continue;
@@ -590,32 +683,30 @@ void BamfLauncherIcon::Focus(ActionArg arg)
       windows.push_back(xid);
     }
 
-    if (WindowManager::Default()->IsWindowOnCurrentDesktop(xid) &&
-        WindowManager::Default()->IsWindowVisible(xid))
+    if (wm->IsWindowOnCurrentDesktop(xid) && wm->IsWindowVisible(xid))
     {
       any_visible = true;
     }
   }
+
   g_list_free(children);
 
+  auto visibility = WindowManager::FocusVisibility::OnlyVisible;
+
   if (arg.source != ActionArg::SWITCHER)
   {
     if (any_visible)
     {
-      WindowManager::Default()->FocusWindowGroup(windows,
-       WindowManager::FocusVisibility::ForceUnminimizeInvisible, arg.monitor);
+      visibility = WindowManager::FocusVisibility::ForceUnminimizeInvisible;
     }
     else
     {
-      WindowManager::Default()->FocusWindowGroup(windows,
-       WindowManager::FocusVisibility::ForceUnminimizeOnCurrentDesktop, arg.monitor);
+      visibility = WindowManager::FocusVisibility::ForceUnminimizeOnCurrentDesktop;
     }
   }
-  else
-  {
-    WindowManager::Default()->FocusWindowGroup(windows,
-      WindowManager::FocusVisibility::OnlyVisible, arg.monitor);
-  }
+
+  bool only_top_win = !any_urgent;
+  wm->FocusWindowGroup(windows, visibility, arg.monitor, only_top_win);
 }
 
 bool BamfLauncherIcon::Spread(bool current_desktop, int state, bool force)
@@ -679,8 +770,8 @@ void BamfLauncherIcon::UpdateDesktopQuickList()
   const gchar** nicks = indicator_desktop_shortcuts_get_nicks(_desktop_shortcuts);
 
   int index = 0;
-  while (nicks[index]) {
-
+  while (nicks[index])
+  {
     // Build a dbusmenu item for each nick that is the desktop
     // file that is built from it's name and includes a callback
     // to the desktop shortcuts object to execute the nick
@@ -946,7 +1037,13 @@ std::list<DbusmenuMenuitem*> BamfLauncherIcon::GetMenus()
 
     _gsignals.Add(new glib::Signal<void, DbusmenuMenuitem*, int>(item, DBUSMENU_MENUITEM_SIGNAL_ITEM_ACTIVATED,
                                     [&] (DbusmenuMenuitem*, int) {
-                                      ActivateLauncherIcon(ActionArg(ActionArg::OTHER, 0));
+                                      _quicklist_activated_id =
+                                        g_idle_add([] (gpointer data) -> gboolean {
+                                          auto self = static_cast<BamfLauncherIcon*>(data);
+                                          self->ActivateLauncherIcon(ActionArg());
+                                          self->_quicklist_activated_id = 0;
+                                          return FALSE;
+                                        }, this);
                                     }));
 
     _menu_items_extra["AppName"] = glib::Object<DbusmenuMenuitem>(item);
@@ -1054,31 +1151,12 @@ std::string BamfLauncherIcon::GetRemoteUri()
   return _remote_uri;
 }
 
-std::set<std::string> BamfLauncherIcon::ValidateUrisForLaunch(unity::DndData& uris)
+std::set<std::string> BamfLauncherIcon::ValidateUrisForLaunch(DndData const& uris)
 {
   std::set<std::string> result;
-  gboolean is_home_launcher = g_str_has_suffix(DesktopFile().c_str(), "nautilus-home.desktop");
 
-  if (is_home_launcher)
-  {
-    for (auto k : uris.Uris())
-      result.insert(k);
-    return result;
-  }
-
-  for (auto i : uris.Types())
-  {
-    for (auto j : GetSupportedTypes())
-    {
-      if (g_content_type_is_a(i.c_str(), j.c_str()))
-      {
-        for (auto k : uris.UrisByType(i))
-          result.insert(k);
-
-        break;
-      }
-    }
-  }
+  for (auto uri : uris.Uris())
+    result.insert(uri);
 
   return result;
 }
@@ -1110,12 +1188,37 @@ void BamfLauncherIcon::OnDndLeave()
   _dnd_hover_timer = 0;
 }
 
-nux::DndAction BamfLauncherIcon::OnQueryAcceptDrop(unity::DndData& dnd_data)
+bool BamfLauncherIcon::OnShouldHighlightOnDrag(DndData const& dnd_data)
+{
+  bool is_home_launcher = boost::algorithm::ends_with(DesktopFile(), "nautilus-home.desktop") ||
+                          boost::algorithm::ends_with(DesktopFile(), "nautilus.desktop");
+
+  if (is_home_launcher)
+  {
+    return true;
+  }
+
+  for (auto type : dnd_data.Types())
+  {
+    for (auto supported_type : GetSupportedTypes())
+    {
+      if (g_content_type_is_a(type.c_str(), supported_type.c_str()))
+      {
+        if (!dnd_data.UrisByType(type).empty())
+          return true;
+      }
+    }
+  }
+
+  return false; 
+}
+
+nux::DndAction BamfLauncherIcon::OnQueryAcceptDrop(DndData const& dnd_data)
 {
   return ValidateUrisForLaunch(dnd_data).empty() ? nux::DNDACTION_NONE : nux::DNDACTION_COPY;
 }
 
-void BamfLauncherIcon::OnAcceptDrop(unity::DndData& dnd_data)
+void BamfLauncherIcon::OnAcceptDrop(DndData const& dnd_data)
 {
   OpenInstanceWithUris(ValidateUrisForLaunch(dnd_data));
 }
