@@ -32,6 +32,7 @@
 #include "DashStyle.h"
 #include "DashSettings.h"
 #include "UBusMessages.h"
+#include "KeyboardUtil.h"
 
 namespace unity
 {
@@ -110,6 +111,10 @@ DashView::~DashView()
     g_source_remove (searching_timeout_id_);
   if (hide_message_delay_id_)
     g_source_remove(hide_message_delay_id_);
+
+  // Do this explicitely, otherwise dee will complain about invalid access
+  // to the lens models
+  RemoveLayout ();
 }
 
 void DashView::SetMonitorOffset(int x, int y)
@@ -315,7 +320,16 @@ void DashView::DrawContent(nux::GraphicsEngine& gfx_context, bool force_draw)
 
 void DashView::OnMouseButtonDown(int x, int y, unsigned long button, unsigned long key)
 {
-  if (!content_geo_.IsPointInside(x, y))
+  dash::Style& style = dash::Style::Instance();
+  nux::Geometry geo(content_geo_);
+
+  if (Settings::Instance().GetFormFactor() == FormFactor::DESKTOP)
+  {
+    geo.width += style.GetDashRightTileWidth();
+    geo.height += style.GetDashBottomTileHeight();
+  }
+
+  if (!geo.IsPointInside(x, y))
   {
     ubus_manager_.SendMessage(UBUS_PLACE_VIEW_CLOSE_REQUEST);
   }
@@ -329,12 +343,21 @@ void DashView::OnActivateRequest(GVariant* args)
 
   g_variant_get(args, "(sus)", &uri, &handled_type, &search_string);
 
-  std::string id = AnalyseLensURI(uri.Str());
+  std::string id(AnalyseLensURI(uri.Str()));
 
-  lens_bar_->Activate(id);
-
-  if ((id == "home.lens" && handled_type != GOTO_DASH_URI ) || !visible_)
+  if (!visible_)
+  {
+    lens_bar_->Activate(id);
     ubus_manager_.SendMessage(UBUS_DASH_EXTERNAL_ACTIVATION);
+  }
+  else if (/* visible_ && */ handled_type == NOT_HANDLED)
+  {
+    ubus_manager_.SendMessage(UBUS_PLACE_VIEW_CLOSE_REQUEST);
+  }
+  else if (/* visible_ && */ handled_type == GOTO_DASH_URI)
+  {
+    lens_bar_->Activate(id);
+  }
 }
 
 std::string DashView::AnalyseLensURI(std::string const& uri)
@@ -737,6 +760,7 @@ std::string DashView::GetName() const
 
 void DashView::AddProperties(GVariantBuilder* builder)
 {
+  dash::Style& style = dash::Style::Instance();
   int num_rows = 1; // The search bar
 
   if (active_lens_view_)
@@ -750,8 +774,11 @@ void DashView::AddProperties(GVariantBuilder* builder)
     form_factor = "desktop";
 
   unity::variant::BuilderWrapper wrapper(builder);
+  wrapper.add(nux::Geometry(GetAbsoluteX(), GetAbsoluteY(), content_geo_.width, content_geo_.height));
   wrapper.add("num_rows", num_rows);
   wrapper.add("form_factor", form_factor);
+  wrapper.add("right-border-width", style.GetDashRightTileWidth());
+  wrapper.add("bottom-border-height", style.GetDashBottomTileHeight());
 }
 
 nux::Area* DashView::KeyNavIteration(nux::KeyNavDirection direction)
@@ -778,15 +805,15 @@ void DashView::ProcessDndEnter()
 }
 
 Area* DashView::FindKeyFocusArea(unsigned int key_symbol,
-      unsigned long x11_key_code,
-      unsigned long special_keys_state)
+                                 unsigned long x11_key_code,
+                                 unsigned long special_keys_state)
 {
   // Do what nux::View does, but if the event isn't a key navigation,
   // designate the text entry to process it.
 
+  nux::KeyNavDirection direction = KEY_NAV_NONE;
   bool ctrl = (special_keys_state & NUX_STATE_CTRL);
 
-  nux::KeyNavDirection direction = KEY_NAV_NONE;
   switch (x11_key_code)
   {
   case NUX_VK_UP:
@@ -824,7 +851,6 @@ Area* DashView::FindKeyFocusArea(unsigned int key_symbol,
     break;
   default:
     direction = KEY_NAV_NONE;
-    break;
   }
 
   // We should not do it here, but I really don't want to make DashView
@@ -832,7 +858,7 @@ Area* DashView::FindKeyFocusArea(unsigned int key_symbol,
   // DashView::KeyNavIteration.
    nux::InputArea* focus_area = nux::GetWindowCompositor().GetKeyFocusArea();
 
-  if (key_symbol == nux::NUX_KEYDOWN && !search_bar_->im_preedit)
+  if (direction != KEY_NAV_NONE && key_symbol == nux::NUX_KEYDOWN && !search_bar_->im_preedit)
   {
     std::list<nux::Area*> tabs;
     for (auto category : active_lens_view_->categories())
@@ -907,7 +933,18 @@ Area* DashView::FindKeyFocusArea(unsigned int key_symbol,
     }
   }
 
-  if (direction == KEY_NAV_NONE || search_bar_->im_preedit)
+  bool search_key = false;
+
+  if (direction == KEY_NAV_NONE)
+  {
+    if (ui::KeyboardUtil::IsPrintableKeySymbol(x11_key_code) ||
+        ui::KeyboardUtil::IsMoveKeySymbol(x11_key_code))
+    {
+      search_key = true;
+    }
+  }
+
+  if (search_key || search_bar_->im_preedit)
   {
     // then send the event to the search entry
     return search_bar_->text_entry();
@@ -916,7 +953,8 @@ Area* DashView::FindKeyFocusArea(unsigned int key_symbol,
   {
     return next_object_to_key_focus_area_->FindKeyFocusArea(key_symbol, x11_key_code, special_keys_state);
   }
-  return NULL;
+
+  return nullptr;
 }
 
 }
