@@ -25,9 +25,8 @@
 #include <UnityCore/GLibWrapper.h>
 #include <UnityCore/Variant.h>
 #include <Nux/HLayout.h>
+#include <Nux/VLayout.h>
 
-#include <NuxCore/Logger.h>
-#include "HudButton.h"
 #include "UBusMessages.h"
 #include "DashStyle.h"
 
@@ -38,25 +37,27 @@ namespace hud
 
 namespace
 {
+
 nux::logging::Logger logger("unity.hud.view");
-const int icon_size = 46;
+
 const int grow_anim_length = 90 * 1000;
 const int pause_before_grow_length = 32 * 1000;
 
-const int default_width = 1024;
+const int default_width = 960;
 const int default_height = 276;
-const int content_width = 941;
+const int content_width = 939;
 
 const int top_padding = 11;
-const int bottom_padding = 9;
+const int bottom_padding = 10;
 const int left_padding = 11;
 const int right_padding = 0;
+
 }
 
 NUX_IMPLEMENT_OBJECT_TYPE(View);
 
 View::View()
-  : nux::View(NUX_TRACKER_LOCATION)
+  : AbstractView()
   , button_views_(NULL)
   , timeline_id_(0)
   , start_time_(0)
@@ -71,11 +72,6 @@ View::View()
     QueueDraw();
   });
 
-  nux::ROPConfig rop;
-  rop.Blend = true;
-  rop.SrcBlend = GL_ONE;
-  rop.DstBlend = GL_ONE_MINUS_SRC_ALPHA;
-
   SetupViews();
   search_bar_->key_down.connect (sigc::mem_fun (this, &View::OnKeyDown));
 
@@ -126,6 +122,9 @@ View::~View()
   {
     RemoveChild((*button).GetPointer());
   }
+
+  if (timeline_id_)
+    g_source_remove(timeline_id_);
 }
 
 void View::ProcessGrowShrink()
@@ -155,6 +154,11 @@ void View::ProcessGrowShrink()
    current_height_ = new_height;
   }
 
+  for (auto button : buttons_)
+  {
+    button->SetSkipDraw((button->GetAbsoluteY() + button->GetBaseHeight()) > (GetAbsoluteY() + current_height_));
+  }
+
   QueueDraw();
 
   if (diff > grow_anim_length + pause_before_grow_length)
@@ -169,13 +173,16 @@ void View::ProcessGrowShrink()
 
 void View::ResetToDefault()
 {
+  SetQueries(Hud::Queries());
+  current_height_ = content_layout_->GetBaseHeight();
+
   search_bar_->search_string = "";
   search_bar_->search_hint = _("Type your command");
 }
 
 void View::Relayout()
 {
-  nux::Geometry geo = GetGeometry();
+  nux::Geometry const& geo = GetGeometry();
   content_geo_ = GetBestFitGeometry(geo);
   LOG_DEBUG(logger) << "content_geo: " << content_geo_.width << "x" << content_geo_.height;
 
@@ -211,6 +218,12 @@ nux::View* View::default_focus() const
   return search_bar_->text_entry();
 }
 
+std::list<HudButton::Ptr> const& View::buttons() const
+{
+  return buttons_;
+}
+
+
 void View::SetQueries(Hud::Queries queries)
 {
   // early exit, if the user is key navigating on the hud, we don't want to set new
@@ -229,14 +242,16 @@ void View::SetQueries(Hud::Queries queries)
   buttons_.clear();
   button_views_->Clear();
   int found_items = 0;
-  for (auto query = queries.begin(); query != queries.end(); query++)
+  for (auto query : queries)
   {
     if (found_items >= 5)
       break;
 
     HudButton::Ptr button(new HudButton());
     buttons_.push_front(button);
-    button->SetQuery(*query);
+    button->SetMinimumWidth(content_width);
+    button->SetMaximumWidth(content_width);
+    button->SetQuery(query);
 
     button_views_->AddView(button.GetPointer(), 0, nux::MINOR_POSITION_LEFT);
 
@@ -244,33 +259,43 @@ void View::SetQueries(Hud::Queries queries)
       query_activated.emit(dynamic_cast<HudButton*>(view)->GetQuery());
     });
 
-    button->key_nav_focus_activate.connect([&](nux::Area *area) {
+    button->key_nav_focus_activate.connect([&](nux::Area* area) {
       query_activated.emit(dynamic_cast<HudButton*>(area)->GetQuery());
     });
 
-    button->key_nav_focus_change.connect([&](nux::Area *area, bool recieving, KeyNavDirection direction){
+    button->key_nav_focus_change.connect([&](nux::Area* area, bool recieving, KeyNavDirection direction){
       if (recieving)
         query_selected.emit(dynamic_cast<HudButton*>(area)->GetQuery());
     });
 
-    // You should never decrement end().  We should fix this loop.
-    button->is_rounded = (query == --(queries.end())) ? true : false;
-    button->fake_focused = (query == (queries.begin())) ? true : false;
-
-    button->SetMinimumWidth(content_width);
-    found_items++;
+    ++found_items;
   }
+
   if (found_items)
+  {
+    buttons_.front()->is_rounded = true;
+    buttons_.back()->fake_focused = true;
     selected_button_ = 1;
+  }
+
 
   QueueRelayout();
   QueueDraw();
 }
 
-void View::SetIcon(std::string icon_name)
+void View::SetIcon(std::string const& icon_name, unsigned int tile_size, unsigned int size, unsigned int padding)
 {
+  if (!icon_)
+    return;
+
   LOG_DEBUG(logger) << "Setting icon to " << icon_name;
-  icon_->SetByIconName(icon_name, icon_size);
+
+  icon_->SetIcon(icon_name, size, tile_size, padding);
+
+  /* We need to compute this value manually, since the _content_layout height changes */
+  int content_height = search_bar_->GetBaseHeight() + top_padding + bottom_padding;
+  icon_->SetMinimumHeight(std::max(icon_->GetMinimumHeight(), content_height));
+
   QueueDraw();
 }
 
@@ -307,10 +332,8 @@ nux::Geometry View::GetBestFitGeometry(nux::Geometry const& for_geo)
   int width = default_width;
   int height = default_height;
 
-  if (!show_embedded_icon_)
-  {
-    width -= icon_->GetGeometry().width;
-  }
+  if (show_embedded_icon_)
+    width += icon_->GetGeometry().width;
 
   LOG_DEBUG (logger) << "best fit is, " << width << ", " << height;
 
@@ -339,11 +362,11 @@ void View::SetupViews()
 {
   dash::Style& style = dash::Style::Instance();
 
-  nux::VLayout* super_layout = new nux::VLayout(); 
+  nux::VLayout* super_layout = new nux::VLayout();
   layout_ = new nux::HLayout();
   {
     // fill layout with icon
-    icon_ = new Icon("", icon_size);
+    icon_ = new Icon();
     {
       AddChild(icon_.GetPointer());
       layout_->AddView(icon_.GetPointer(), 0, nux::MINOR_POSITION_LEFT, nux::MINOR_SIZE_FULL);
@@ -361,7 +384,7 @@ void View::SetupViews()
       search_bar_->SetMinimumHeight(style.GetSearchBarHeight());
       search_bar_->SetMaximumHeight(style.GetSearchBarHeight());
       search_bar_->search_hint = _("Type your command");
-      search_bar_->search_changed.connect(sigc::mem_fun(this, &View::OnSearchChanged));
+      search_bar_->live_search_reached.connect(sigc::mem_fun(this, &View::OnSearchChanged));
       AddChild(search_bar_.GetPointer());
       content_layout_->AddView(search_bar_.GetPointer(), 0, nux::MINOR_POSITION_LEFT);
 
@@ -382,21 +405,12 @@ void View::OnSearchChanged(std::string const& search_string)
 {
   LOG_DEBUG(logger) << "got search change";
   search_changed.emit(search_string);
-  if (search_string.empty())
+
+  for (auto button : buttons_)
   {
-    search_bar_->search_hint = _("Type your command");
-  }
-  else
-  {
-    search_bar_->search_hint = "";
+    button->fake_focused = false;
   }
 
-  std::list<HudButton::Ptr>::iterator it;
-  for(it = buttons_.begin(); it != buttons_.end(); ++it)
-  {
-    (*it)->fake_focused = false;
-  }
-  
   if (!buttons_.empty())
     buttons_.back()->fake_focused = true;
 }
@@ -415,7 +429,9 @@ void View::OnKeyDown (unsigned long event_type, unsigned long keysym,
 
 void View::OnMouseButtonDown(int x, int y, unsigned long button, unsigned long key)
 {
-  if (!content_geo_.IsPointInside(x, y))
+  nux::Geometry current_geo(content_geo_);
+  current_geo.height = current_height_;
+  if (!current_geo.IsPointInside(x, y))
   {
     ubus.SendMessage(UBUS_HUD_CLOSE_REQUEST);
   }
@@ -441,9 +457,21 @@ void View::DrawContent(nux::GraphicsEngine& gfx_context, bool force_draw)
   renderer_.DrawInner(gfx_context, draw_content_geo, absolute_window_geometry_, window_geometry_);
 
   gfx_context.PushClippingRectangle(draw_content_geo);
+
   if (IsFullRedraw())
   {
     nux::GetPainter().PushBackgroundStack();
+
+    if (!buttons_.empty()) // See bug #1008603.
+    {
+      int height = 3;
+      int x = search_bar_->GetBaseX() + 1;
+      int y = search_bar_->GetBaseY() + search_bar_->GetBaseHeight() - height;
+      nux::GetPainter().Draw2DLine(gfx_context, x, y, x, y + height, nux::color::White * 0.13);
+      x += content_width - 1;
+      nux::GetPainter().Draw2DLine(gfx_context, x, y, x, y + height, nux::color::White * 0.13);
+    }
+
     GetLayout()->ProcessDraw(gfx_context, force_draw);
     nux::GetPainter().PopBackgroundStack();
   }
@@ -457,7 +485,7 @@ void View::DrawContent(nux::GraphicsEngine& gfx_context, bool force_draw)
 
   if (timeline_need_more_draw_ && !timeline_id_)
   {
-    timeline_id_ = g_timeout_add(0, [] (gpointer data) -> gboolean
+    timeline_id_ = g_idle_add([] (gpointer data) -> gboolean
     {
       View *self = static_cast<View*>(data);
       self->QueueDraw();
@@ -483,7 +511,7 @@ void View::AddProperties(GVariantBuilder* builder)
 {
   unsigned num_buttons = buttons_.size();
   variant::BuilderWrapper(builder)
-    .add(GetGeometry())
+    .add(content_geo_)
     .add("selected_button", selected_button_)
     .add("num_buttons", num_buttons);
 }
@@ -501,13 +529,17 @@ bool View::InspectKeyEvent(unsigned int eventType,
     else
     {
       search_bar_->search_string = "";
-      search_bar_->search_hint = _("Type your command");
     }
     return true;
   }
   return false;
 }
 
+void View::SearchFinished()
+{
+  search_bar_->SearchFinished();
+}
+
 void View::OnSearchbarActivated()
 {
   // The "Enter" key has been received and the text entry has the key focus.
@@ -571,13 +603,11 @@ nux::Area* View::FindKeyFocusArea(unsigned int event_type,
 
     if (search_bar_->search_string == "")
     {
-      search_bar_->search_hint = _("Type your command");
       ubus.SendMessage(UBUS_HUD_CLOSE_REQUEST);
     }
     else
     {
       search_bar_->search_string = "";
-      search_bar_->search_hint = _("Type your command");
       return search_bar_->text_entry();
     }
     return NULL;
