@@ -140,6 +140,8 @@ const int Launcher::Launcher::ANIM_DURATION_SHORT = 125;
 Launcher::Launcher(nux::BaseWindow* parent,
                    NUX_FILE_LINE_DECL)
   : View(NUX_FILE_LINE_PARAM)
+  , _launcher_drag_delta_max(0)
+  , _launcher_drag_delta_min(0)
   , _model(0)
   , _collection_window(NULL)
   , _background_color(nux::color::DimGray)
@@ -200,7 +202,6 @@ Launcher::Launcher(nux::BaseWindow* parent,
   _folded_angle           = 1.0f;
   _neg_folded_angle       = -1.0f;
   _space_between_icons    = 5;
-  _last_delta_y           = 0.0f;
   _folded_z_distance      = 10.0f;
   _launcher_action_state  = ACTION_NONE;
   _icon_under_mouse       = NULL;
@@ -225,7 +226,6 @@ Launcher::Launcher(nux::BaseWindow* parent,
   _shortcuts_shown        = false;
   _hovered                = false;
   _hidden                 = false;
-  _scroll_limit_reached   = false;
   _render_drag_window     = false;
   _drag_edge_touching     = false;
   _steal_drag             = false;
@@ -294,6 +294,16 @@ Launcher::Launcher(nux::BaseWindow* parent,
   launcher_pressure_effect_ = cache.FindTexture("launcher_pressure_effect", 0, 0, cb);
 
   options.changed.connect (sigc::mem_fun (this, &Launcher::OnOptionsChanged));
+
+  // icon information from software center
+  _sc_icon = NULL;
+  _sc_icon_title = NULL;
+  _sc_icon_x = 0;
+  _sc_icon_y = 0;
+  _sc_icon_size = 0;
+  _sc_icon_desktop_file = NULL;
+  _sc_icon_aptdaemon_task = NULL;
+  _sc_anim_icon = false;
 }
 
 Launcher::~Launcher()
@@ -321,6 +331,11 @@ Launcher::~Launcher()
 
   delete _hover_machine;
   delete _hide_machine;
+
+  g_free(_sc_icon);
+  g_free(_sc_icon_title);
+  g_free(_sc_icon_desktop_file);
+  g_free(_sc_icon_aptdaemon_task);
 }
 
 /* Introspection */
@@ -396,6 +411,7 @@ Launcher::AddProperties(GVariantBuilder* builder)
   .add("hovered", _hovered)
   .add("hidemode", options()->hide_mode)
   .add("hidden", _hidden)
+  .add("is_showing", ! _hidden)
   .add("x", abs_geo.x)
   .add("y", abs_geo.y)
   .add("width", abs_geo.width)
@@ -1186,28 +1202,28 @@ void Launcher::RenderArgs(std::list<RenderArg> &launcher_args,
   last_geo = box_geo;
   _enter_y = 0;
 
+  // logically dnd exit only restores to the clamped ranges
+  // hover_progress restores to 0
+  _launcher_drag_delta_max = 0.0f;
+  _launcher_drag_delta_min = std::min(0.0f, launcher_height - sum);
+
   if (hover_progress > 0.0f && _launcher_drag_delta != 0)
   {
     float delta_y = _launcher_drag_delta;
 
-    // logically dnd exit only restores to the clamped ranges
-    // hover_progress restores to 0
-    float max = 0.0f;
-    float min = MIN(0.0f, launcher_height - sum);
-
-    if (_launcher_drag_delta > max)
-      delta_y = max + DragLimiter(delta_y - max);
-    else if (_launcher_drag_delta < min)
-      delta_y = min + DragLimiter(delta_y - min);
+    if (_launcher_drag_delta > _launcher_drag_delta_max)
+      delta_y = _launcher_drag_delta_max + DragLimiter(delta_y - _launcher_drag_delta_max);
+    else if (_launcher_drag_delta < _launcher_drag_delta_min)
+      delta_y = _launcher_drag_delta_min + DragLimiter(delta_y - _launcher_drag_delta_min);
 
     if (GetActionState() != ACTION_DRAG_LAUNCHER)
     {
       float dnd_progress = DnDExitProgress(current);
 
-      if (_launcher_drag_delta > max)
-        delta_y = max + (delta_y - max) * dnd_progress;
-      else if (_launcher_drag_delta < min)
-        delta_y = min + (delta_y - min) * dnd_progress;
+      if (_launcher_drag_delta > _launcher_drag_delta_max)
+        delta_y = _launcher_drag_delta_max + (delta_y - _launcher_drag_delta_max) * dnd_progress;
+      else if (_launcher_drag_delta < _launcher_drag_delta_min)
+        delta_y = _launcher_drag_delta_min + (delta_y - _launcher_drag_delta_min) * dnd_progress;
 
       if (dnd_progress == 0.0f)
         _launcher_drag_delta = (int) delta_y;
@@ -1216,9 +1232,6 @@ void Launcher::RenderArgs(std::list<RenderArg> &launcher_args,
     delta_y *= hover_progress;
     center.y += delta_y;
     folding_threshold += delta_y;
-
-    _scroll_limit_reached = (delta_y == _last_delta_y);
-    _last_delta_y = delta_y;
   }
   else
   {
@@ -1339,7 +1352,7 @@ void Launcher::OnOverlayShown(GVariant* data)
   gint32 overlay_monitor = 0;
   g_variant_get(data, UBUS_OVERLAY_FORMAT_STRING,
                 &overlay_identity, &can_maximise, &overlay_monitor);
-  std::string identity = overlay_identity.Str();
+  std::string identity(overlay_identity.Str());
 
   LOG_DEBUG(logger) << "Overlay shown: " << identity
                     << ", " << (can_maximise ? "can maximise" : "can't maximise")
@@ -1365,6 +1378,9 @@ void Launcher::OnOverlayShown(GVariant* data)
       LOG_DEBUG(logger) << "Desaturate on monitor " << monitor();
       DesaturateIcons();
     }
+
+    if (_icon_under_mouse)
+      _icon_under_mouse->HideTooltip();
   }
   EnsureAnimation();
 }
@@ -1636,6 +1652,16 @@ bool Launcher::IsBackLightModeToggles() const
   }
 }
 
+nux::ObjectPtr<nux::View> const& Launcher::GetActiveTooltip() const
+{
+  return _active_tooltip;
+}
+
+nux::ObjectPtr<LauncherDragWindow> Launcher::GetDraggedIcon() const
+{
+  return nux::ObjectPtr<LauncherDragWindow>(_drag_window);
+}
+
 void
 Launcher::SetActionState(LauncherActionState actionstate)
 {
@@ -1705,44 +1731,46 @@ bool Launcher::MouseOverBottomScrollExtrema()
 gboolean Launcher::OnScrollTimeout(gpointer data)
 {
   Launcher* self = (Launcher*) data;
-  nux::Geometry geo = self->GetGeometry();
-  gboolean anim = TRUE;
+  gboolean continue_animation = TRUE;
 
-  //
-  // Always check _scroll_limit_reached to ensure we don't keep spinning
-  // this timer if the mouse happens to be left idle over one of the autoscroll
-  // hotspots on the launcher.
-  //
   if (self->IsInKeyNavMode() || !self->_hovered ||
-      self->_scroll_limit_reached ||
       self->GetActionState() == ACTION_DRAG_LAUNCHER)
-    anim = FALSE;
+  {
+    continue_animation = FALSE;
+  }
   else if (self->MouseOverTopScrollArea())
   {
-    if (self->MouseOverTopScrollExtrema())
+    if (self->_launcher_drag_delta >= self->_launcher_drag_delta_max)
+      continue_animation = FALSE;
+    else if (self->MouseOverTopScrollExtrema())
       self->_launcher_drag_delta += 6;
     else
       self->_launcher_drag_delta += 3;
   }
   else if (self->MouseOverBottomScrollArea())
   {
-    if (self->MouseOverBottomScrollExtrema())
+    if (self->_launcher_drag_delta <= self->_launcher_drag_delta_min)
+      continue_animation = FALSE;
+    else if (self->MouseOverBottomScrollExtrema())
       self->_launcher_drag_delta -= 6;
     else
       self->_launcher_drag_delta -= 3;
   }
   else
-    anim = FALSE;
+  {
+    continue_animation = FALSE;
+  }
 
-  if (anim)
+  if (continue_animation != FALSE)
+  {
     self->EnsureAnimation();
+  }
   else
   {
     self->_autoscroll_handle = 0;
-    self->_scroll_limit_reached = false;
   }
 
-  return anim;
+  return continue_animation;
 }
 
 void Launcher::EnsureScrollTimer()
@@ -1796,6 +1824,7 @@ void Launcher::OnIconAdded(AbstractLauncherIcon::Ptr icon)
   EnsureAnimation();
 
   icon->needs_redraw.connect(sigc::mem_fun(this, &Launcher::OnIconNeedsRedraw));
+  icon->tooltip_visible.connect(sigc::mem_fun(this, &Launcher::OnTooltipVisible));
 }
 
 void Launcher::OnIconRemoved(AbstractLauncherIcon::Ptr icon)
@@ -1872,6 +1901,11 @@ void Launcher::OnIconNeedsRedraw(AbstractLauncherIcon::Ptr icon)
   EnsureAnimation();
 }
 
+void Launcher::OnTooltipVisible(nux::ObjectPtr<nux::View> view)
+{
+  _active_tooltip = view;
+}
+
 void Launcher::Draw(nux::GraphicsEngine& GfxContext, bool force_draw)
 {
 
@@ -2005,7 +2039,7 @@ void Launcher::DrawContent(nux::GraphicsEngine& GfxContext, bool force_draw)
 
     // apply the bg colour
 #ifndef NUX_OPENGLES_20
-    if (GfxContext.UsingGLSLCodePath() == FALSE)
+    if (GfxContext.UsingGLSLCodePath() == false)
       gPainter.Paint2DQuadColor(GfxContext, bkg_box, _background_color);
 #endif
 
@@ -2083,6 +2117,16 @@ void Launcher::DrawContent(nux::GraphicsEngine& GfxContext, bool force_draw)
   gPainter.PopBackground(push_count);
   GfxContext.PopClippingRectangle();
   GfxContext.PopClippingRectangle();
+
+  if (_sc_anim_icon)
+  {
+    launcher_addrequest_special.emit(_sc_icon_desktop_file, AbstractLauncherIcon::Ptr(), _sc_icon_aptdaemon_task, _sc_icon, _sc_icon_x, _sc_icon_y, _sc_icon_size);
+    g_free(_sc_icon); _sc_icon = NULL;
+    g_free(_sc_icon_title); _sc_icon_title = NULL;
+    g_free(_sc_icon_desktop_file); _sc_icon_desktop_file = NULL;
+    g_free(_sc_icon_aptdaemon_task); _sc_icon_aptdaemon_task = NULL;
+    _sc_anim_icon = false;
+  }
 }
 
 void Launcher::PostDraw(nux::GraphicsEngine& GfxContext, bool force_draw)
@@ -2243,8 +2287,36 @@ void Launcher::UpdateDragWindowPosition(int x, int y)
 
       if (progress >= 1.0f)
         _model->ReorderSmart(_drag_icon, hovered_icon, true);
-      else if (progress == 0.0f)
-        _model->ReorderBefore(_drag_icon, hovered_icon, false);
+      else if (progress == 0.0f) {
+        if (_drag_icon->GetIconType() == hovered_icon->GetIconType()) {
+          _model->ReorderBefore(_drag_icon, hovered_icon, false);
+        } else {
+          // LauncherModel::ReorderBefore does not work on different icon types
+          // so if hovered_icon is of a different type than _drag_icon
+          // try to use LauncherModel::ReorderAfter with the icon that is before hovered_icon
+          AbstractLauncherIcon::Ptr iconBeforeHover;
+          LauncherModel::iterator it;
+          LauncherModel::iterator prevIt = _model->end();
+          for (it = _model->begin(); it != _model->end(); it++)
+          {
+            if (!(*it)->GetQuirk(AbstractLauncherIcon::QUIRK_VISIBLE) || !(*it)->IsVisibleOnMonitor(monitor))
+              continue;
+
+            if ((*it) == hovered_icon) {
+              if (prevIt != _model->end()) {
+                iconBeforeHover = *prevIt;
+              }
+              break;
+            }
+
+            prevIt = it;
+          }
+
+          if (iconBeforeHover && _drag_icon != iconBeforeHover) {
+            _model->ReorderAfter(_drag_icon, iconBeforeHover);
+          }
+        }
+      }
     }
   }
 }
@@ -2320,7 +2392,14 @@ void Launcher::RecvMouseDrag(int x, int y, int dx, int dy, unsigned long button_
     }
     else
     {
-      StartIconDragRequest(x, y);
+      // We we can safely start the icon drag, from the original mouse-down position
+      if (_start_dragicon_handle > 0)
+      {
+        g_source_remove(_start_dragicon_handle);
+        _start_dragicon_handle = 0;
+      }
+
+      StartIconDragRequest(x - _dnd_delta_x, y - _dnd_delta_y);
     }
   }
   else if (GetActionState() == ACTION_DRAG_LAUNCHER)
@@ -2533,7 +2612,7 @@ void Launcher::MouseDownLogic(int x, int y, unsigned long button_flags, unsigned
       g_source_remove(_start_dragicon_handle);
     _start_dragicon_handle = g_timeout_add(START_DRAGICON_DURATION, &Launcher::StartIconDragTimeout, this);
 
-    launcher_icon->mouse_down.emit(nux::GetEventButton(button_flags), monitor);
+    launcher_icon->mouse_down.emit(nux::GetEventButton(button_flags), key_flags, monitor);
   }
 }
 
@@ -2549,17 +2628,20 @@ void Launcher::MouseUpLogic(int x, int y, unsigned long button_flags, unsigned l
 
   if (_icon_mouse_down && (_icon_mouse_down == launcher_icon))
   {
-    _icon_mouse_down->mouse_up.emit(nux::GetEventButton(button_flags), monitor);
+    _icon_mouse_down->mouse_up.emit(nux::GetEventButton(button_flags), key_flags, monitor);
 
     if (GetActionState() == ACTION_NONE)
     {
-      _icon_mouse_down->mouse_click.emit(nux::GetEventButton(button_flags), monitor);
+
+      /* This will inform the icon if the action is valid for all the monitors */
+      int action_monitor = options()->show_for_all ? -1 : monitor;
+      _icon_mouse_down->mouse_click.emit(nux::GetEventButton(button_flags), key_flags, action_monitor);
     }
   }
 
   if (launcher_icon && (_icon_mouse_down != launcher_icon))
   {
-    launcher_icon->mouse_up.emit(nux::GetEventButton(button_flags), monitor);
+    launcher_icon->mouse_up.emit(nux::GetEventButton(button_flags), key_flags, monitor);
   }
 
   if (GetActionState() == ACTION_DRAG_LAUNCHER)
@@ -2663,14 +2745,8 @@ void Launcher::OnDNDDataCollected(const std::list<char*>& mimes)
     break;
   }
 
-  if (!_dnd_data.Uris().size())
-    return;
-
   _hide_machine->SetQuirk(LauncherHideMachine::EXTERNAL_DND_ACTIVE, true);
 
-  if (IsOverlayOpen())
-    SaturateIcons();
-
   for (auto it : _dnd_data.Uris())
   {
     if (g_str_has_suffix(it.c_str(), ".desktop"))
@@ -2684,12 +2760,23 @@ void Launcher::OnDNDDataCollected(const std::list<char*>& mimes)
   {
     for (auto it : *_model)
     {
-      if (it->QueryAcceptDrop(_dnd_data) != nux::DNDACTION_NONE)
+      if (it->ShouldHighlightOnDrag(_dnd_data))
+      {
+        it->SetQuirk(AbstractLauncherIcon::QUIRK_DESAT, false);
         it->SetQuirk(AbstractLauncherIcon::QUIRK_DROP_PRELIGHT, true);
+        it->SetQuirk(AbstractLauncherIcon::QUIRK_PRESENTED, true);
+      }
       else
+      {
         it->SetQuirk(AbstractLauncherIcon::QUIRK_DROP_DIM, true);
+      }
     }
   }
+  else
+  {
+    if (IsOverlayOpen())
+      SaturateIcons();
+  }
 }
 
 void
@@ -2714,6 +2801,7 @@ Launcher::DndReset()
   {
     it->SetQuirk(AbstractLauncherIcon::QUIRK_DROP_PRELIGHT, false);
     it->SetQuirk(AbstractLauncherIcon::QUIRK_DROP_DIM, false);
+    it->SetQuirk(AbstractLauncherIcon::QUIRK_PRESENTED, false);
   }
 
   DndHoveredIconReset();
@@ -2784,7 +2872,7 @@ Launcher::ProcessDndMove(int x, int y, std::list<char*> mimes)
     {
       for (auto it : *_model)
       {
-        if (it->QueryAcceptDrop(_dnd_data) != nux::DNDACTION_NONE)
+        if (it->ShouldHighlightOnDrag(_dnd_data))
           it->SetQuirk(AbstractLauncherIcon::QUIRK_DROP_PRELIGHT, true);
         else
           it->SetQuirk(AbstractLauncherIcon::QUIRK_DROP_DIM, true);
@@ -2912,6 +3000,9 @@ Launcher::ProcessDndDrop(int x, int y)
   }
   else if (_dnd_hovered_icon && _drag_action != nux::DNDACTION_NONE)
   {
+    if (IsOverlayOpen())
+      ubus.SendMessage(UBUS_PLACE_VIEW_CLOSE_REQUEST);
+
     _dnd_hovered_icon->AcceptDrop(_dnd_data);
   }
 
@@ -2948,30 +3039,25 @@ Launcher::handle_dbus_method_call(GDBusConnection*       connection,
                                   GDBusMethodInvocation* invocation,
                                   gpointer               user_data)
 {
-
   if (g_strcmp0(method_name, "AddLauncherItemFromPosition") == 0)
   {
-    gchar*  icon;
-    gchar*  title;
-    gint32  icon_x;
-    gint32  icon_y;
-    gint32  icon_size;
-    gchar*  desktop_file;
-    gchar*  aptdaemon_task;
-
-    g_variant_get(parameters, "(ssiiiss)", &title, &icon, &icon_x, &icon_y, &icon_size, &desktop_file, &aptdaemon_task, NULL);
-
     Launcher* self = (Launcher*)user_data;
-    self->launcher_addrequest_special.emit(desktop_file, AbstractLauncherIcon::Ptr(), aptdaemon_task, icon,
-                                            icon_x, icon_y, icon_size);
+    self->_sc_anim_icon = true;
+    g_free(self->_sc_icon);
+    g_free(self->_sc_icon_desktop_file);
+    g_free(self->_sc_icon_aptdaemon_task);
+    g_variant_get(parameters, "(ssiiiss)", &self->_sc_icon_title,
+      &self->_sc_icon,
+      &self->_sc_icon_x,
+      &self->_sc_icon_y,
+      &self->_sc_icon_size,
+      &self->_sc_icon_desktop_file,
+      &self->_sc_icon_aptdaemon_task, NULL);
 
     g_dbus_method_invocation_return_value(invocation, nullptr);
-    g_free(icon);
-    g_free(title);
-    g_free(desktop_file);
-    g_free(aptdaemon_task);
-  }
 
+    self->EnsureAnimation();
+  }
 }
 
 void
