@@ -18,7 +18,6 @@
  *              Tim Penhey <tim.penhey@canonical.com>
  */
 
-#include <gio/gio.h>
 #include <glib/gi18n-lib.h>
 #include <libbamf/libbamf.h>
 
@@ -27,28 +26,21 @@
 #include <Nux/BaseWindow.h>
 #include <NuxCore/Logger.h>
 
-#include "LauncherOptions.h"
 #include "BamfLauncherIcon.h"
+#include "BFBLauncherIcon.h"
 #include "DesktopLauncherIcon.h"
 #include "DeviceLauncherIcon.h"
-#include "DeviceLauncherSection.h"
-#include "EdgeBarrierController.h"
 #include "FavoriteStore.h"
 #include "HudLauncherIcon.h"
-#include "Launcher.h"
 #include "LauncherController.h"
-#include "LauncherEntryRemote.h"
-#include "LauncherEntryRemoteModel.h"
-#include "AbstractLauncherIcon.h"
+#include "LauncherControllerPrivate.h"
+#include "LauncherOptions.h"
 #include "SoftwareCenterLauncherIcon.h"
-#include "LauncherModel.h"
-#include "WindowManager.h"
-#include "TrashLauncherIcon.h"
-#include "BFBLauncherIcon.h"
-#include "UScreen.h"
-#include "UBusWrapper.h"
-#include "UBusMessages.h"
 #include "TimeUtil.h"
+#include "TrashLauncherIcon.h"
+#include "UBusMessages.h"
+#include "UScreen.h"
+#include "WindowManager.h"
 
 namespace unity
 {
@@ -70,120 +62,6 @@ namespace
 }
 }
 
-class Controller::Impl
-{
-public:
-  Impl(Display* display, Controller* parent);
-  ~Impl();
-
-  void UpdateNumWorkspaces(int workspaces);
-
-  Launcher* CreateLauncher(int monitor);
-
-  void Save();
-  void SortAndUpdate();
-
-  nux::ObjectPtr<Launcher> CurrentLauncher();
-
-  void OnIconAdded(AbstractLauncherIcon::Ptr icon);
-  void OnIconRemoved(AbstractLauncherIcon::Ptr icon);
-
-  void OnLauncherAddRequest(char* path, AbstractLauncherIcon::Ptr before);
-  void OnLauncherAddRequestSpecial(std::string const& path, AbstractLauncherIcon::Ptr before, std::string const& aptdaemon_trans_id, std::string const& icon_path,
-                                   int icon_x, int icon_y, int icon_size);
-  void OnLauncherRemoveRequest(AbstractLauncherIcon::Ptr icon);
-  void OnSCIconAnimationComplete(AbstractLauncherIcon::Ptr icon);
-
-  void OnLauncherEntryRemoteAdded(LauncherEntryRemote::Ptr const& entry);
-  void OnLauncherEntryRemoteRemoved(LauncherEntryRemote::Ptr const& entry);
-
-  void OnFavoriteStoreFavoriteAdded(std::string const& entry, std::string const& pos, bool before);
-  void OnFavoriteStoreFavoriteRemoved(std::string const& entry);
-  void OnFavoriteStoreReordered();
-
-
-  void InsertExpoAction();
-  void RemoveExpoAction();
-
-  void InsertDesktopIcon();
-  void RemoveDesktopIcon();
-
-  void SendHomeActivationRequest();
-
-  int MonitorWithMouse();
-
-  void InsertTrash();
-
-  void RegisterIcon(AbstractLauncherIcon::Ptr icon);
-
-  AbstractLauncherIcon::Ptr CreateFavorite(const char* file_path);
-
-  SoftwareCenterLauncherIcon::Ptr CreateSCLauncherIcon(std::string const& file_path, std::string const& aptdaemon_trans_id, std::string const& icon_path);
-
-  void SetupBamf();
-
-  void EnsureLaunchers(int primary, std::vector<nux::Geometry> const& monitors);
-
-  void OnExpoActivated();
-
-  void OnScreenChanged(int primary_monitor, std::vector<nux::Geometry>& monitors);
-
-  void OnWindowFocusChanged (guint32 xid);
-
-  void ReceiveMouseDownOutsideArea(int x, int y, unsigned long button_flags, unsigned long key_flags);
-
-  void ReceiveLauncherKeyPress(unsigned long eventType,
-                               unsigned long keysym,
-                               unsigned long state,
-                               const char* character,
-                               unsigned short keyCount);
-
-  /* statics */
-
-  static void OnViewOpened(BamfMatcher* matcher, BamfView* view, gpointer data);
-
-  Controller* parent_;
-  glib::Object<BamfMatcher> matcher_;
-  LauncherModel::Ptr     model_;
-  nux::ObjectPtr<Launcher> launcher_;
-  nux::ObjectPtr<Launcher> keyboard_launcher_;
-  int                    sort_priority_;
-  DeviceLauncherSection* device_section_;
-  LauncherEntryRemoteModel remote_model_;
-  AbstractLauncherIcon::Ptr expo_icon_;
-  AbstractLauncherIcon::Ptr desktop_launcher_icon_;
-  AbstractLauncherIcon::Ptr desktop_icon_;
-  int                    num_workspaces_;
-  bool                   show_desktop_icon_;
-  Display*               display_;
-
-  guint                  bamf_timer_handler_id_;
-  guint                  on_view_opened_id_;
-  guint                  launcher_key_press_handler_id_;
-  guint                  launcher_label_show_handler_id_;
-  guint                  launcher_hide_handler_id_;
-
-  bool                   launcher_open;
-  bool                   launcher_keynav;
-  bool                   launcher_grabbed;
-  bool                   reactivate_keynav;
-  int                    reactivate_index;
-  bool                   keynav_restore_window_;
-
-  UBusManager            ubus;
-
-  int                    launcher_key_press_time_;
-
-  ui::EdgeBarrierController::Ptr edge_barriers_;
-
-  LauncherList launchers;
-
-  sigc::connection on_expoicon_activate_connection_;
-  sigc::connection launcher_key_press_connection_;
-  sigc::connection launcher_event_outside_connection_;
-};
-
-
 Controller::Impl::Impl(Display* display, Controller* parent)
   : parent_(parent)
   , matcher_(nullptr)
@@ -194,9 +72,8 @@ Controller::Impl::Impl(Display* display, Controller* parent)
   , launcher_key_press_handler_id_(0)
   , launcher_label_show_handler_id_(0)
   , launcher_hide_handler_id_(0)
-  , edge_barriers_(new ui::EdgeBarrierController())
 {
-  edge_barriers_->options = parent_->options();
+  edge_barriers_.options = parent_->options();
 
   UScreen* uscreen = UScreen::GetDefault();
   auto monitors = uscreen->GetMonitors();
@@ -211,9 +88,7 @@ Controller::Impl::Impl(Display* display, Controller* parent)
   EnsureLaunchers(primary, monitors);
 
   launcher_ = launchers[0];
-
-  device_section_ = new DeviceLauncherSection();
-  device_section_->IconAdded.connect(sigc::mem_fun(this, &Impl::OnIconAdded));
+  device_section_.IconAdded.connect(sigc::mem_fun(this, &Impl::OnIconAdded));
 
   num_workspaces_ = WindowManager::Default()->WorkspaceCount();
   if (num_workspaces_ > 1)
@@ -290,11 +165,6 @@ Controller::Impl::~Impl()
 
   if (bamf_timer_handler_id_ != 0)
     g_source_remove(bamf_timer_handler_id_);
-
-  if (matcher_ != nullptr && on_view_opened_id_ != 0)
-    g_signal_handler_disconnect((gpointer) matcher_, on_view_opened_id_);
-
-  delete device_section_;
 }
 
 void Controller::Impl::EnsureLaunchers(int primary, std::vector<nux::Geometry> const& monitors)
@@ -302,54 +172,43 @@ void Controller::Impl::EnsureLaunchers(int primary, std::vector<nux::Geometry> c
   unsigned int num_monitors = monitors.size();
   unsigned int num_launchers = parent_->multiple_launchers ? num_monitors : 1;
   unsigned int launchers_size = launchers.size();
-  unsigned int last_monitor = 0;
+  unsigned int last_launcher = 0;
 
-  if (num_launchers == 1)
+  for (unsigned int i = 0; i < num_launchers; i++, last_launcher++)
   {
-    if (launchers_size == 0)
+    if (i >= launchers_size)
     {
-      launchers.push_back(nux::ObjectPtr<Launcher>(CreateLauncher(primary)));
+      launchers.push_back(nux::ObjectPtr<Launcher>(CreateLauncher(i)));
     }
-    else if (!launchers[0].IsValid())
+    else if (!launchers[i])
     {
-      launchers[0] = nux::ObjectPtr<Launcher>(CreateLauncher(primary));
+      launchers[i] = nux::ObjectPtr<Launcher>(CreateLauncher(i));
     }
 
-    launchers[0]->monitor(primary);
-    launchers[0]->Resize();
-    last_monitor = 1;
-  }
-  else
-  {
-    for (unsigned int i = 0; i < num_monitors; i++, last_monitor++)
-    {
-      if (i >= launchers_size)
-      {
-        launchers.push_back(nux::ObjectPtr<Launcher>(CreateLauncher(i)));
-      }
+    int monitor = (num_launchers == 1) ? primary : i;
 
-      launchers[i]->monitor(i);
-      launchers[i]->Resize();
+    if (launchers[i]->monitor() != monitor)
+    {
+      edge_barriers_.Unsubscribe(launchers[i].GetPointer(), launchers[i]->monitor);
     }
+
+    launchers[i]->monitor(monitor);
+    launchers[i]->Resize();
+    edge_barriers_.Subscribe(launchers[i].GetPointer(), launchers[i]->monitor);
   }
 
-  for (unsigned int i = last_monitor; i < launchers_size; ++i)
+  for (unsigned int i = last_launcher; i < launchers_size; ++i)
   {
     auto launcher = launchers[i];
-    if (launcher.IsValid())
+    if (launcher)
     {
       parent_->RemoveChild(launcher.GetPointer());
       launcher->GetParent()->UnReference();
-      edge_barriers_->Unsubscribe(launcher.GetPointer(), launcher->monitor);
+      edge_barriers_.Unsubscribe(launcher.GetPointer(), launcher->monitor);
     }
   }
 
   launchers.resize(num_launchers);
-
-  for (size_t i = 0; i < launchers.size(); ++i)
-  {
-    edge_barriers_->Subscribe(launchers[i].GetPointer(), launchers[i]->monitor);
-  }
 }
 
 void Controller::Impl::OnScreenChanged(int primary_monitor, std::vector<nux::Geometry>& monitors)
@@ -381,6 +240,7 @@ Launcher* Controller::Impl::CreateLauncher(int monitor)
   launcher->display = display_;
   launcher->monitor = monitor;
   launcher->options = parent_->options();
+  launcher->SetModel(model_.get());
 
   nux::HLayout* layout = new nux::HLayout(NUX_TRACKER_LOCATION);
   layout->AddView(launcher, 1);
@@ -395,7 +255,6 @@ Launcher* Controller::Impl::CreateLauncher(int monitor)
   launcher_window->InputWindowEnableStruts(false);
   launcher_window->SetEnterFocusInputArea(launcher);
 
-  launcher->SetModel(model_.get());
   launcher->launcher_addrequest.connect(sigc::mem_fun(this, &Impl::OnLauncherAddRequest));
   launcher->launcher_addrequest_special.connect(sigc::mem_fun(this, &Impl::OnLauncherAddRequestSpecial));
   launcher->launcher_removerequest.connect(sigc::mem_fun(this, &Impl::OnLauncherRemoveRequest));
@@ -467,6 +326,11 @@ Controller::Impl::OnLauncherAddRequestSpecial(std::string const& path,
       return;
   }
 
+ // Check if desktop file was supplied, or if it's set to SC's agent
+ // See https://bugs.launchpad.net/unity/+bug/1002440
+  if (path.empty() || path == "software-center-agent")
+    return;
+
   SoftwareCenterLauncherIcon::Ptr result = CreateSCLauncherIcon(path, aptdaemon_trans_id, icon_path);
 
   launcher_->ForceReveal(true);
@@ -498,7 +362,7 @@ void Controller::Impl::SortAndUpdate()
       std::stringstream shortcut_string;
       shortcut_string << (shortcut % 10);
       icon->SetShortcut(shortcut_string.str()[0]);
-      shortcut++;
+      ++shortcut;
     }
     // reset shortcut
     else
@@ -608,6 +472,8 @@ void Controller::Impl::OnFavoriteStoreFavoriteAdded(std::string const& entry, st
     else
       model_->ReorderBefore(result, other, false);
   }
+
+  SortAndUpdate();
 }
 
 void Controller::Impl::OnFavoriteStoreFavoriteRemoved(std::string const& entry)
@@ -700,13 +566,12 @@ void Controller::Impl::RemoveExpoAction()
 
 void Controller::Impl::InsertDesktopIcon()
 {
-  desktop_launcher_icon_ = AbstractLauncherIcon::Ptr(new DesktopLauncherIcon());
-  RegisterIcon(desktop_launcher_icon_);
+  RegisterIcon(desktop_icon_);
 }
 
 void Controller::Impl::RemoveDesktopIcon()
 {
-  model_->RemoveIcon(desktop_launcher_icon_);
+  model_->RemoveIcon(desktop_icon_);
 }
 
 void Controller::Impl::RegisterIcon(AbstractLauncherIcon::Ptr icon)
@@ -724,15 +589,12 @@ void Controller::Impl::RegisterIcon(AbstractLauncherIcon::Ptr icon)
 }
 
 /* static private */
-void Controller::Impl::OnViewOpened(BamfMatcher* matcher, BamfView* view, gpointer data)
+void Controller::Impl::OnViewOpened(BamfMatcher* matcher, BamfView* view)
 {
-  Impl* self = static_cast<Impl*>(data);
-  BamfApplication* app;
-
   if (!BAMF_IS_APPLICATION(view))
     return;
 
-  app = BAMF_APPLICATION(view);
+  BamfApplication* app = BAMF_APPLICATION(view);
 
   if (bamf_view_is_sticky(view) ||
       g_object_get_qdata(G_OBJECT(app), g_quark_from_static_string("unity-seen")))
@@ -740,10 +602,11 @@ void Controller::Impl::OnViewOpened(BamfMatcher* matcher, BamfView* view, gpoint
     return;
   }
 
-  AbstractLauncherIcon::Ptr icon (new BamfLauncherIcon(app));
-  icon->SetSortPriority(self->sort_priority_++);
-
-  self->RegisterIcon(icon);
+  AbstractLauncherIcon::Ptr icon(new BamfLauncherIcon(app));
+  icon->visibility_changed.connect(sigc::mem_fun(this, &Impl::SortAndUpdate));
+  icon->SetSortPriority(sort_priority_++);
+  RegisterIcon(icon);
+  SortAndUpdate();
 }
 
 AbstractLauncherIcon::Ptr Controller::Impl::CreateFavorite(const char* file_path)
@@ -761,8 +624,6 @@ AbstractLauncherIcon::Ptr Controller::Impl::CreateFavorite(const char* file_path
     return result;
   }
 
-  g_object_set_qdata(G_OBJECT(app), g_quark_from_static_string("unity-seen"), GINT_TO_POINTER(1));
-
   bamf_view_set_sticky(BAMF_VIEW(app), true);
   AbstractLauncherIcon::Ptr icon (new BamfLauncherIcon(app));
   icon->SetSortPriority(sort_priority_++);
@@ -822,7 +683,7 @@ void Controller::Impl::SetupBamf()
   }
 
   apps = bamf_matcher_get_applications(matcher_);
-  on_view_opened_id_ = g_signal_connect(matcher_, "view-opened", (GCallback) &Impl::OnViewOpened, this);
+  view_opened_signal_.Connect(matcher_, "view-opened", sigc::mem_fun(this, &Impl::OnViewOpened));
 
   for (l = apps; l; l = l->next)
   {
@@ -830,7 +691,6 @@ void Controller::Impl::SetupBamf()
 
     if (g_object_get_qdata(G_OBJECT(app), g_quark_from_static_string("unity-seen")))
       continue;
-    g_object_set_qdata(G_OBJECT(app), g_quark_from_static_string("unity-seen"), GINT_TO_POINTER(1));
 
     AbstractLauncherIcon::Ptr icon(new BamfLauncherIcon(app));
     icon->SetSortPriority(sort_priority_++);
@@ -847,7 +707,7 @@ void Controller::Impl::SetupBamf()
 
 void Controller::Impl::SendHomeActivationRequest()
 {
-  ubus.SendMessage(UBUS_PLACE_ENTRY_ACTIVATE_REQUEST, g_variant_new("(sus)", "home.lens", 0, ""));
+  ubus.SendMessage(UBUS_PLACE_ENTRY_ACTIVATE_REQUEST, g_variant_new("(sus)", "home.lens", dash::NOT_HANDLED, ""));
 }
 
 Controller::Controller(Display* display)
@@ -904,9 +764,16 @@ std::vector<AbstractLauncherIcon::Ptr> Controller::GetAltTabIcons(bool current)
   results.push_back(pimpl->desktop_icon_);
 
   for (auto icon : *(pimpl->model_))
+  {
     if (icon->ShowInSwitcher(current))
-      results.push_back(icon);
-
+    {
+      //otherwise we get two desktop icons in the switcher.
+      if (icon->GetIconType() != AbstractLauncherIcon::IconType::TYPE_DESKTOP)
+      {
+        results.push_back(icon);
+      }
+    }
+  }
   return results;
 }
 
@@ -1204,7 +1071,13 @@ void Controller::KeyNavTerminate(bool activate)
   }
 
   if (activate)
-    pimpl->model_->Selection()->Activate(ActionArg(ActionArg::LAUNCHER, 0));
+  {
+    g_idle_add([] (gpointer data) -> gboolean {
+      Controller *self = (Controller*)data;
+        self->pimpl->model_->Selection()->Activate(ActionArg(ActionArg::LAUNCHER, 0));
+      return FALSE;
+    }, this);
+  }
 
   pimpl->launcher_keynav = false;
   if (!pimpl->launcher_open)
