@@ -50,7 +50,6 @@ public:
 
   void Init();
   void OnBackgroundColorChanged(GVariant* args);
-  void OnDashDecorationDamanged(GVariant* args);
 
   void Draw(nux::GraphicsEngine& gfx_context, nux::Geometry content_geo, nux::Geometry absolute_geo, nux::Geometry geometry, bool force_draw);
   void DrawContent(nux::GraphicsEngine& gfx_context, nux::Geometry content_geo, nux::Geometry absolute_geo, nux::Geometry geometry);
@@ -117,9 +116,6 @@ void OverlayRendererImpl::Init()
   ubus_manager_.RegisterInterest(UBUS_BACKGROUND_COLOR_CHANGED,
                                  sigc::mem_fun(this, &OverlayRendererImpl::OnBackgroundColorChanged));
 
-  ubus_manager_.RegisterInterest(UBUS_DASH_DECORATION_DAMAGED,
-                                 sigc::mem_fun(this, &OverlayRendererImpl::OnDashDecorationDamanged));
-
   ubus_manager_.SendMessage(UBUS_BACKGROUND_REQUEST_COLOUR_EMIT);
 }
 
@@ -135,11 +131,6 @@ void OverlayRendererImpl::OnBackgroundColorChanged(GVariant* args)
   parent->need_redraw.emit();
 }
 
-void OverlayRendererImpl::OnDashDecorationDamanged(GVariant* args)
-{
-  parent->need_redraw.emit();
-}
-
 void OverlayRendererImpl::InitASMInverseTextureMaskShader()
 {
   nux::NString AsmVtx = 
@@ -190,6 +181,7 @@ void OverlayRendererImpl::InitASMInverseTextureMaskShader()
 
 void OverlayRendererImpl::RenderInverseMask_ASM(nux::GraphicsEngine& gfx_context, int x, int y, int width, int height, nux::ObjectPtr<nux::IOpenGLBaseTexture> device_texture, nux::TexCoordXForm &texxform, const nux::Color &color)
 {
+#ifndef NUX_OPENGLES_20
   if (!inverse_texture_mask_asm_prog_.IsValid() || !inverse_texture_rect_mask_asm_prog_.IsValid())
   {
     InitASMInverseTextureMaskShader();
@@ -255,6 +247,7 @@ void OverlayRendererImpl::RenderInverseMask_ASM(nux::GraphicsEngine& gfx_context
     CHECKGL(glDisableVertexAttribArrayARB(VertexColorLocation));
 
   shader_program->End();
+#endif
 }
 
 void OverlayRendererImpl::InitSlInverseTextureMaskShader()
@@ -385,14 +378,15 @@ void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry c
   bool paint_blur = BackgroundEffectHelper::blur_type != BLUR_NONE;
   nux::Geometry geo(content_geo);
 
-  
+  int excess_border = (dash::Settings::Instance().GetFormFactor() != dash::FormFactor::NETBOOK || force_edges) ? EXCESS_BORDER : 0;
+
   nux::Geometry larger_content_geo = content_geo;
-  larger_content_geo.OffsetSize(EXCESS_BORDER, EXCESS_BORDER);
+  larger_content_geo.OffsetSize(excess_border, excess_border);
 
   nux::Geometry larger_geo(larger_content_geo);
 
   nux::Geometry larger_absolute_geo = absolute_geo;
-  larger_absolute_geo.OffsetSize(EXCESS_BORDER, EXCESS_BORDER);
+  larger_absolute_geo.OffsetSize(excess_border, excess_border);
 
   nux::TexCoordXForm texxform_absolute_bg;
   texxform_absolute_bg.flip_v_coord = true;
@@ -498,14 +492,13 @@ void OverlayRendererImpl::Draw(nux::GraphicsEngine& gfx_context, nux::Geometry c
   nux::GetPainter().RenderSinglePaintLayer(gfx_context, larger_content_geo, bg_darken_layer_);
 
 #ifndef NUX_OPENGLES_20
-  if (gfx_context.UsingGLSLCodePath() == FALSE)
+  if (gfx_context.UsingGLSLCodePath() == false)
   {
     bg_layer_->SetGeometry(larger_content_geo);
     nux::GetPainter().RenderSinglePaintLayer(gfx_context, larger_content_geo, bg_layer_);
   }
 #endif
 
-
   texxform_absolute_bg.flip_v_coord = false;
   texxform_absolute_bg.uoffset = (1.0f / bg_shine_texture_->GetWidth()) * parent->x_offset;
   texxform_absolute_bg.voffset = (1.0f / bg_shine_texture_->GetHeight()) * parent->y_offset;
@@ -774,13 +767,15 @@ void OverlayRendererImpl::DrawContent(nux::GraphicsEngine& gfx_context, nux::Geo
   nux::Geometry geo = geometry;
   bgs = 0;
 
+  int excess_border = (dash::Settings::Instance().GetFormFactor() != dash::FormFactor::NETBOOK) ? EXCESS_BORDER : 0;
+
   nux::Geometry larger_content_geo = content_geo;
-  larger_content_geo.OffsetSize(EXCESS_BORDER, EXCESS_BORDER);
+  larger_content_geo.OffsetSize(excess_border, excess_border);
 
   nux::Geometry larger_geo(larger_content_geo);
 
   nux::Geometry larger_absolute_geo = absolute_geo;
-  larger_absolute_geo.OffsetSize(EXCESS_BORDER, EXCESS_BORDER);
+  larger_absolute_geo.OffsetSize(excess_border, excess_border);
   
 
   gfx_context.PushClippingRectangle(larger_geo);
@@ -833,7 +828,7 @@ void OverlayRendererImpl::DrawContent(nux::GraphicsEngine& gfx_context, nux::Geo
   bgs++;
 
 #ifndef NUX_OPENGLES_20
-  if (gfx_context.UsingGLSLCodePath() == FALSE)
+  if (gfx_context.UsingGLSLCodePath() == false)
   {
     nux::GetPainter().PushLayer(gfx_context, bg_layer_->GetGeometry(), bg_layer_);
     bgs++;
