@@ -20,8 +20,10 @@
 #include <glib.h>
 #include <sstream>
 #include "PluginAdapter.h"
+#include "UScreen.h"
 
 #include <NuxCore/Logger.h>
+#include <UnityCore/Variant.h>
 
 namespace
 {
@@ -392,6 +394,12 @@ PluginAdapter::InitiateExpo()
 }
 
 // WindowManager implementation
+guint32
+PluginAdapter::GetActiveWindow()
+{
+  return m_Screen->activeWindow();
+}
+
 bool
 PluginAdapter::IsWindowMaximized(guint xid)
 {
@@ -521,6 +529,58 @@ PluginAdapter::IsWindowVisible(guint32 xid)
   return false;
 }
 
+bool
+PluginAdapter::IsWindowOnTop(guint32 xid)
+{
+  Window win = xid;
+  CompWindow* window = m_Screen->findWindow(win);
+
+  if (window)
+  {
+    if (window->inShowDesktopMode() || !window->isMapped() || !window->isViewable() || window->minimized())
+      return false;
+
+    CompPoint window_vp = window->defaultViewport();
+    nux::Geometry const& window_vp_geo = GetWorkAreaGeometry(window->id());
+    std::vector<Window> const& our_xids = nux::XInputWindow::NativeHandleList();
+
+    for (CompWindow* sibling = window->next; sibling; sibling = sibling->next)
+    {
+      if (sibling->defaultViewport() == window_vp && !sibling->minimized() &&
+          sibling->isMapped() && sibling->isViewable() && !sibling->inShowDesktopMode() &&
+          !(sibling->state() & CompWindowStateAboveMask) &&
+          !(sibling->type() & CompWindowTypeSplashMask) &&
+          !(sibling->type() & CompWindowTypeDockMask) &&
+          /* FIXME: This should be included by the above defaultViewport() check,
+           * but it doesn't seem to work correctly when there's only one workspace
+           * enabled, so please drop the line above when bug #996604 is fixed in
+           * Compiz. */
+          !window_vp_geo.Intersect(GetWindowGeometry(sibling->id())).IsNull() &&
+          std::find(our_xids.begin(), our_xids.end(), sibling->id()) == our_xids.end())
+      {
+        return false;
+      }
+    }
+
+    return true;
+  }
+
+  return false;
+}
+
+bool
+PluginAdapter::IsWindowClosable(guint32 xid)
+{
+  Window win = xid;
+  CompWindow* window;
+
+  window = m_Screen->findWindow(win);
+  if (window)
+    return (window->actions() & CompWindowActionCloseMask);
+
+  return false;
+}
+
 bool
 PluginAdapter::IsWindowMinimizable(guint32 xid)
 {
@@ -534,6 +594,19 @@ PluginAdapter::IsWindowMinimizable(guint32 xid)
   return false;
 }
 
+bool
+PluginAdapter::IsWindowMaximizable(guint32 xid)
+{
+  Window win = xid;
+  CompWindow* window;
+
+  window = m_Screen->findWindow(win);
+  if (window)
+    return (window->actions() & MAXIMIZABLE);
+
+  return false;
+}
+
 void
 PluginAdapter::Restore(guint32 xid)
 {
@@ -618,11 +691,12 @@ PluginAdapter::Lower(guint32 xid)
 }
 
 void 
-PluginAdapter::FocusWindowGroup(std::vector<Window> window_ids, FocusVisibility focus_visibility, int monitor)
+PluginAdapter::FocusWindowGroup(std::vector<Window> window_ids, FocusVisibility focus_visibility, int monitor, bool only_top_win)
 {
   CompPoint target_vp = m_Screen->vp();
-  CompWindow* top_window = NULL;
-  CompWindow* top_window_on_monitor = NULL;
+  CompWindow* top_window = nullptr;
+  CompWindow* top_monitor_win = nullptr;
+
   bool any_on_current = false;
   bool any_mapped = false;
   bool any_mapped_on_current = false;
@@ -661,11 +735,12 @@ PluginAdapter::FocusWindowGroup(std::vector<Window> window_ids, FocusVisibility
 
   if (!any_on_current)
   {
-    for (auto it = windows.rbegin(); it != windows.rend(); it++)
+    for (auto it = windows.rbegin(); it != windows.rend(); ++it)
     {
-      if ((any_mapped && !(*it)->minimized()) || !any_mapped)
+      CompWindow* win = *it;
+      if ((any_mapped && !win->minimized()) || !any_mapped)
       {
-        target_vp = (*it)->defaultViewport();
+        target_vp = win->defaultViewport();
         break;
       }
     }
@@ -675,46 +750,64 @@ PluginAdapter::FocusWindowGroup(std::vector<Window> window_ids, FocusVisibility
   {
     if (win->defaultViewport() == target_vp)
     {
-       /* Any window which is actually unmapped is
-        * not going to be accessible by either switcher
-        * or scale, so unconditionally unminimize those
-        * windows when the launcher icon is activated */
-       if ((focus_visibility == WindowManager::FocusVisibility::ForceUnminimizeOnCurrentDesktop &&
-            target_vp == m_Screen->vp()) ||
-            (focus_visibility == WindowManager::FocusVisibility::ForceUnminimizeInvisible &&
-             win->mapNum () == 0))
-       {
-         bool is_mapped = win->mapNum () != 0;
-         top_window = win;
-         if (monitor >= 0 && win->outputDevice() == monitor)
-          top_window_on_monitor = win;
-         win->unminimize ();
+      int win_monitor = GetWindowMonitor(win->id());
 
-         forced_unminimize = true;
+      /* Any window which is actually unmapped is
+      * not going to be accessible by either switcher
+      * or scale, so unconditionally unminimize those
+      * windows when the launcher icon is activated */
+      if ((focus_visibility == WindowManager::FocusVisibility::ForceUnminimizeOnCurrentDesktop &&
+          target_vp == m_Screen->vp()) ||
+          (focus_visibility == WindowManager::FocusVisibility::ForceUnminimizeInvisible &&
+           win->mapNum() == 0))
+      {
+        top_window = win;
+        forced_unminimize = true;
 
-         /* Initially minimized windows dont get raised */
-         if (!is_mapped)
-           win->raise ();
-       }
-       else if ((any_mapped_on_current && !win->minimized()) || !any_mapped_on_current)
-       {
-         if (!forced_unminimize || target_vp == m_Screen->vp())
-         {
-           win->raise();
-           top_window = win;
-           if (monitor >= 0 && win->outputDevice() == monitor)
-            top_window_on_monitor = win;
-         }
-       }
+        if (monitor >= 0 && win_monitor == monitor)
+          top_monitor_win = win;
+
+        if (!only_top_win)
+        {
+          bool is_mapped = (win->mapNum() != 0);
+          win->unminimize();
+
+           /* Initially minimized windows dont get raised */
+          if (!is_mapped)
+            win->raise();
+        }
+      }
+      else if ((any_mapped_on_current && !win->minimized()) || !any_mapped_on_current)
+      {
+        if (!forced_unminimize || target_vp == m_Screen->vp())
+        {
+          top_window = win;
+
+          if (monitor >= 0 && win_monitor == monitor)
+            top_monitor_win = win;
+
+          if (!only_top_win)
+            win->raise();
+        }
+      }
     }
   }
 
-  if (monitor > 0 && top_window_on_monitor)
-  {
-    top_window_on_monitor->activate();
-  }
-  else if (top_window)
+  if (monitor >= 0 && top_monitor_win)
+    top_window = top_monitor_win;
+
+  if (top_window)
   {
+    if (only_top_win)
+    {
+      if (forced_unminimize)
+        {
+          top_window->unminimize();
+        }
+
+      top_window->raise();
+    }
+
     top_window->activate();
   }
 }
@@ -768,12 +861,29 @@ PluginAdapter::OnLeaveDesktop()
   _in_show_desktop = false;
 }
 
+int
+PluginAdapter::GetWindowMonitor(guint32 xid) const
+{
+  // FIXME, we should use window->outputDevice() but this is not UScreen friendly
+  nux::Geometry const& geo = GetWindowGeometry(xid);
+
+  if (!geo.IsNull())
+  {
+    int x = geo.x + geo.width/2;
+    int y = geo.y + geo.height/2;
+
+    return unity::UScreen::GetDefault()->GetMonitorAtPosition(x, y);
+  }
+
+  return -1;
+}
+
 nux::Geometry
 PluginAdapter::GetWindowGeometry(guint32 xid) const
 {
   Window win = xid;
   CompWindow* window;
-  nux::Geometry geo(0, 0, 1, 1);
+  nux::Geometry geo;
 
   window = m_Screen->findWindow(win);
   if (window)
@@ -1196,3 +1306,16 @@ PluginAdapter::OnWindowClosed(CompWindow *w)
     _last_focused_window = NULL;
 }
 
+void
+PluginAdapter::AddProperties(GVariantBuilder* builder)
+{
+  unity::variant::BuilderWrapper wrapper(builder);
+  wrapper.add(GetScreenGeometry())
+         .add("workspace_count", WorkspaceCount())
+         .add("active_window", GetActiveWindow())
+         .add("screen_grabbed", IsScreenGrabbed())
+         .add("scale_active", IsScaleActive())
+         .add("scale_active_for_group", IsScaleActiveForGroup())
+         .add("expo_active", IsExpoActive())
+         .add("viewport_switch_running", IsViewPortSwitchStarted());
+}
