@@ -126,6 +126,8 @@ SearchBar::SearchBar(NUX_FILE_LINE_DECL)
   , show_filter_hint_(true)
   , expander_view_(nullptr)
   , show_filters_(nullptr)
+  , last_width_(-1)
+  , last_height_(-1)
   , live_search_timeout_(0)
   , start_spinner_timeout_(0)
 {
@@ -140,6 +142,8 @@ SearchBar::SearchBar(bool show_filter_hint_, NUX_FILE_LINE_DECL)
   , show_filter_hint_(show_filter_hint_)
   , expander_view_(nullptr)
   , show_filters_(nullptr)
+  , last_width_(-1)
+  , last_height_(-1)
   , live_search_timeout_(0)
   , start_spinner_timeout_(0)
 {
@@ -298,27 +302,29 @@ SearchBar::~SearchBar()
 
 void SearchBar::OnFontChanged(GtkSettings* settings, GParamSpec* pspec)
 {
-  gchar* font_name = NULL;
+  glib::String font_name;
   PangoFontDescription* desc;
   std::ostringstream font_desc;
 
   g_object_get(settings, "gtk-font-name", &font_name, NULL);
 
-  desc = pango_font_description_from_string(font_name);
-  pango_entry_->SetFontFamily(pango_font_description_get_family(desc));
-  pango_entry_->SetFontSize(PANGO_ENTRY_FONT_SIZE);
-  pango_entry_->SetFontOptions(gdk_screen_get_font_options(gdk_screen_get_default()));
+  desc = pango_font_description_from_string(font_name.Value());
+  if (desc)
+  {
+    pango_entry_->SetFontFamily(pango_font_description_get_family(desc));
+    pango_entry_->SetFontSize(PANGO_ENTRY_FONT_SIZE);
+    pango_entry_->SetFontOptions(gdk_screen_get_font_options(gdk_screen_get_default()));
 
-  font_desc << pango_font_description_get_family(desc) << " " << HINT_LABEL_FONT_STYLE << " " << HINT_LABEL_FONT_SIZE;
-  hint_->SetFont(font_desc.str().c_str());
+    font_desc << pango_font_description_get_family(desc) << " " << HINT_LABEL_FONT_STYLE << " " << HINT_LABEL_FONT_SIZE;
+    hint_->SetFont(font_desc.str().c_str());
 
-  font_desc.str("");
-  font_desc.clear();
-  font_desc << pango_font_description_get_family(desc) << " " << SHOW_FILTERS_LABEL_FONT_STYLE << " " << SHOW_FILTERS_LABEL_FONT_SIZE;
-  show_filters_->SetFont(font_desc.str().c_str());
+    font_desc.str("");
+    font_desc.clear();
+    font_desc << pango_font_description_get_family(desc) << " " << SHOW_FILTERS_LABEL_FONT_STYLE << " " << SHOW_FILTERS_LABEL_FONT_SIZE;
+    show_filters_->SetFont(font_desc.str().c_str());
 
-  pango_font_description_free(desc);
-  g_free(font_name);
+    pango_font_description_free(desc);
+  }
 }
 
 void SearchBar::OnSearchHintChanged()
@@ -516,7 +522,7 @@ void SearchBar::UpdateBackground(bool force)
 {
   int RADIUS = 5;
   nux::Geometry geo(GetGeometry());
-  geo.width = layered_layout_->GetAbsoluteX() + 
+  geo.width = layered_layout_->GetAbsoluteX() +
               layered_layout_->GetAbsoluteWidth() -
               GetAbsoluteX() +
               SEARCH_ENTRY_RIGHT_BORDER;
@@ -661,7 +667,8 @@ void SearchBar::AddProperties(GVariantBuilder* builder)
   .add("filter-label-x", show_filters_->GetAbsoluteX())
   .add("filter-label-y", show_filters_->GetAbsoluteY())
   .add("filter-label-width", show_filters_->GetAbsoluteWidth())
-  .add("filter-label-height", show_filters_->GetAbsoluteHeight());
+  .add("filter-label-height", show_filters_->GetAbsoluteHeight())
+  .add("im_active", pango_entry_->im_active());
 }
 
 } // namespace unity
